'''
  BA_SpiderPaint_v1_179 
  pdb.plug_in_gmic

  Filter to create a number of 'plasmatic web' layers
  from which a nice image may be created.
  Based on one of Lylejk's flows from Gimp Chat.
  Thank you Lyle!
  Also to Dinasset, thank you for the name suggestion!
  Written by Brian Andersen (animicule@gimpchat.com)
  March 2017  
  Filter Dependent upon GMIC vers 1.7.9
  May or may not work using any other known version of GMIC.
  Filter developed on Windows 7 64bit and Gimp Version 2.8.16

 You should have received a copy of the GNU General Public License
 along with this program if not, you can view the GNU General Public
 License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
 Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.
'''
#!/usr/bin/env python
from gimpfu import *
import decimal
import sys

######################
# INTERNAL FUNCTIONS #  
######################
def Decode_A_Layer (LayerString):
   #  LayerID     #Create Y/N x Desat Y/N x Solidify 0-10 x Scale% 0-300.0 x Noise .0-20.0 x SegLen .5-15.0 x Morph 0-10
   #   1 LS[0]      2 LS[1]      3 LS[2]      4 LS[3]        5 LS[4]           6 LS[5]          7 LS[6]      8 LS[7]
    LS = LayerString.lower()
    LS = LS.replace(' ','')
    exitCnt = LS.count('x')
    LS = LS.split('x')

    if LS[1] =='Y' or LS[1] == 'y':    ## if not creating layer then not testing unused string
        if exitCnt < 7: 
            pdb.gimp_message('Command string min 7 values w/ 6 "x" separators. Line#'+' '+str(LS[0]))
            sys.exit()
        if exitCnt > 7:
            pdb.gimp_message('exitCnt>7 Exit'+str(exitCnt)+' Line#'+' '+str(LS[0]))
            sys.exit()       
        ### LS = LS.split('x') working4/1/17
        if len(LS) < 8:
            pdb.gimp_message('Command string under minimum 8 values. Exit'+' Line#'+' '+str(LS[0]))
            sys.exit()
        if len(LS) > 8:
            pdb.gimp_message('LS > 8 :'+str(len(LS))+' Line#'+' '+str(LS[0]))
            sys.exit()
##-------------------
    global makeLayer
    if LS[1] == 'n' :
        makeLayer = False
        return(makeLayer)
    elif LS[1] == 'y':
        makeLayer = True
    else:
        pdb.gimp_message('CREATE option entry not Y or N'+' Line#'+' '+str(LS[0]))
        sys.exit()
##-------------------
    global deSat
    if LS[2] == 'y':
        deSat = True
    elif LS[2] == 'n':
        deSat = False
    else:
        pdb.gimp_message('DESATURATION option entry not Y or N'+' Line#'+' '+str(LS[0]))
        sys.exit()
##-------------------
    global solidifyOption
    if not -1 < int(LS[3]) < 21:
        pdb.gimp_message('LS[3] solidifyOption not between 0 and 21'+' Line#'+' '+str(LS[0]))
        sys.exit()
    else:
        solidifyOption = int(LS[3])
##-------------------
    global layerScale
    if int(LS[4]) == 0:
        layerScale = 1
    elif not 9 < int(LS[4]) < 301:
        pdb.gimp_message('LS[4] layerScale not between 1 and 301.0'+' Line#'+' '+str(LS[0]))
        sys.exit()
    else:
        layerScale = float(LS[4])/100
##-------------------
    global squiggleNoise
    if not 0.0 < float(LS[5]) < 20.01:
        pdb.gimp_mssage('LS[5] squiggleNoise not between 0.0 and 20.01'+' Line#'+' '+str(LS[0]))
        sys.exit()
    else:
        squiggleNoise = float(LS[5])
##-------------------
    global segmentLength
    if not 0.49 < float(LS[6]) < 15.01:
        pdb.gimp_message('LS[6] segmentLength not between 0.5 and 15.0'+' Line#'+' '+str(LS[0]))
        sys.exit()
    else:
        segmentLength = float(LS[6])
##-------------------
    global morphAmount
    if not -1 < int(LS[7]) < 12:
        pdb.gimp_message('LS[7] morphAmount not between 0 and 10'+' Line#'+' '+str(LS[0]))
        sys.exit()
    else:
        morphAmount = int(LS[7])
##-------------------
    return(makeLayer,deSat,solidifyOption,layerScale,squiggleNoise,segmentLength, morphAmount)
### END FUNCTION DECODE_A_LAYER <<<<----------------------------------------------- 

def Solidify_A_Layer(solidifyOption,worklayer):
    if not 0 < solidifyOption < 21:
        pdb.gimp_message('Solidify Option not 1 to 20'+' Line#'+' '+str(LS[0]))
        return()                               
    if solidifyOption   ==  1:   
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 50,1,0,0,0")     
    elif solidifyOption ==  2:
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 70,1,0,-2,0")    
    elif solidifyOption ==  3:
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 50,2,0,-1,0")    
    elif solidifyOption ==  4:
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 40,1,30,1,0")   
    elif solidifyOption ==  5:  
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 75,1,20,4,0")  
    elif solidifyOption ==  6:                         
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 30,2,45,1,0")  
    elif solidifyOption ==  7:                         
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 35,0,20,0,0")   
    elif solidifyOption ==  8:     
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 70,1,65,0,0") 
    elif solidifyOption ==  9:
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 27,2,76,0,0")   
    elif solidifyOption == 10:
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 50,0,0,3,0")  
    elif solidifyOption == 11:
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 15,0,0,1,0")
    elif solidifyOption == 12:
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 20,1,0,1,0")
    elif solidifyOption == 13:
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 25,2,0,1,0")
    elif solidifyOption == 14:
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 45,1,76,1,0")
    elif solidifyOption == 15:
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 27,2,76,1,0")
    elif solidifyOption == 16:
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 70,1,100,0,0")
    elif solidifyOption == 17:
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 70,2,100,0,0")
    elif solidifyOption == 18:
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 20,0,100,-3,0")
    elif solidifyOption == 19:
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 35,1,20,1,0")
    elif solidifyOption == 20:
        pdb.plug_in_gmic(newImage, worklayer, 1,   "-v - -gimp_solidify_td 50,0,0,3,0")  
    return()                                       
### END FUNCTION Solidify_A_Layer <<<<------------------------------------------- 
def Intensify_Custom(workLay, customIntensity):
    iCnt=0
    while (iCnt < customIntensity):
        TMPLay = workLay.copy()
        TMPLay.opacity = 100.0 
        TMPLay.mode = SCREEN_MODE
        iCnt+=1         
        newImage.add_layer(TMPLay, 0)
        pdb.gimp_item_set_visible(TMPLay, True)
### END FUNCTION Intensify_Custom <<<<-------------------------------------------
def Intensify_inTens(workLay, inTens):
    iCnt=0
    while (iCnt < inTens):
        TMPLay = workLay.copy()
        TMPLay.opacity = 100.0 
        TMPLay.mode = SCREEN_MODE
        iCnt+=1         
        newImage.add_layer(TMPLay, 0)
        pdb.gimp_item_set_visible(TMPLay, True)
### END FUNCTION Intensify_inTens <<<<-------------------------------------------

##################
# MAIN FUNCTION ######################################################
##################        

def BA_SpiderPaint_v1_179(inImage, inDrawable, widthheight, worklen, inTens, lay1str, lay2str, lay3str, lay4str, lay5str, lay6str, lay7str, smokey, lay9str, lay10str, stitched, addbub, weird, ) :

    pdb.gimp_context_push
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()
    width  = inImage.width
    height = inImage.height
    type   = inImage.base_type

    name = inDrawable.name
    if '.' in name:
        split = name.split(".", 1)
        name = split[0]
    suffix = ",BA_SpiderPaint_v1bqt"

   
    global makeLayer, deSat, solidifyOption, layerScale, customIntensity  
    global squiggleNoise, segmentLength, morphAmount, newImage, worklay

    out1=out2=out3=out4=out5=out6=out7=out8=out9=out10=out20=out1a=out2a=out4a=False

    white = "#ffffff"
    black = "#000000"

 ####### calculate work and output scale factors from input

    # calculate outScale factor for the new output image
    if widthheight == 0:   # means the user wants to have same size as input
        outScale = 1
    else:     
        optiWidthHeight = widthheight       # size to work on to get best results
        realWidthHeight = width+height      # real size of the input image
        outScale = decimal.Decimal(optiWidthHeight)/decimal.Decimal(realWidthHeight)  # ratio for scaling

    # layerscale from user input in layerstring scale % calculated in decode layer string
    # layerscale % is applied to output size
    # workscale long side setting start with 1000

    if width > height:
        workScale = decimal.Decimal(worklen)/decimal.Decimal(width)
    else:
        workScale = decimal.Decimal(worklen)/decimal.Decimal(height)

    workwidth  =int(width *workScale)
    workheight =int(height*workScale)
    outwidth  =int(width *outScale)
    outheight =int(height*outScale)

####### create newImage at output size and bring in subject layer
    newImage = pdb.gimp_image_new(outwidth, outheight, type)     
    oriSUB   = pdb.gimp_layer_new_from_drawable(inDrawable, newImage)     #save subject layer
    oriSUB.mode = NORMAL_MODE
    oriSUB.name = name+suffix
    oriSUB.opacity = 100.0 
    newImage.add_layer (oriSUB, 0)
    pdb.gimp_context_set_interpolation(INTERPOLATION_LANCZOS)
    pdb.gimp_layer_scale(oriSUB,outwidth,outheight,False)
    pdb.gimp_layer_set_offsets(oriSUB,0,0) # after scale, re-position layer.
    pdb.gimp_item_set_visible(oriSUB, False)
    alpha=pdb.gimp_drawable_has_alpha(oriSUB)
    if (alpha==False):
        #pdb.gimp_layer_add_alpha (oriSUB)    #  RGBA
        ####### if no alpha layer, no transparency so subject not isolated - end run
        pdb.gimp_message('Subject must be "extracted" with transparent background.\n Script ended Normally')
        return()

####### make pad color layer for stack that will be in screen mode.
    padLayer = pdb.gimp_layer_new(newImage,workwidth,workheight,type,'pad color',100.0,NORMAL_MODE)
    newImage.add_layer(padLayer, 1)  
    pdb.gimp_context_set_foreground(black)
    pdb.gimp_drawable_fill(padLayer,0)
    pdb.gimp_item_set_visible(padLayer,False)

####### Layer 1 Razorsplit Highlight with Intensity #######################################################
    if stitched != 0:  # force creation of lay1 if stitched bg is requested.
        LSx = lay1str.lower()
        LSx = LSx.replace(' ','')
        if LSx[0] == 'n':  # now edit lay1str
            lay1str = 'y'+ LSx[1:]
    LayerString = '1x'+lay1str
    Decode_A_Layer(LayerString)
    if makeLayer:
        out1 = True
        lay1 = oriSUB.copy()
        lay1.mode = NORMAL_MODE
        lay1.name = "RazorSplit Highlight"
        lay1.opacity = 100.0 
        newImage.add_layer(lay1, 0)
        pdb.gimp_item_set_visible(lay1, False)
# scale with layerScale from LayerString ###### Lay1 special layerScale against workwidth not output width 
        if layerScale != workScale:
            pdb.gimp_layer_scale_full(lay1,(workwidth*layerScale),(workheight*layerScale), True,3) # True = local origin, 3=Lanczos
            pdb.gimp_layer_set_offsets(lay1,0,0) # after scale, re-position layer.
# solidify at low scale
        if not solidifyOption == 0:
            Solidify_A_Layer(solidifyOption,lay1)
        if deSat:
            pdb.gimp_desaturate_full(lay1,2)
# squiggle highlight 
        pdb.plug_in_gmic(newImage, lay1, 1,    # 1=input only active layer 
        "-v - -gimp_Squiggly "+str(squiggleNoise)+","+str(segmentLength)+",4,0,.7,0,0,0,3,0.2,0.4,12,1,1,0")
# scale to worksize (worksize) if layerScale != workScale
        if layerScale != workScale:
            pdb.gimp_layer_scale(lay1,workwidth,workheight,False)
            pdb.gimp_layer_set_offsets(lay1,0,0) # after scale, re-position layer.
# morph at normal worksize
        pdb.plug_in_gmic(newImage, lay1, 1,    # 1=input only active layer 
        '-v - -gimp_morpho_v2 0,0,'+str(morphAmount)+','+'"1,0,1; 0,1,0; 1,0,1",0,0,0,1,0')
# intensify custom
        pdb.gimp_item_set_visible(lay1,True)
        Intensify_Custom(lay1,40)
        lay1 = pdb.gimp_image_merge_visible_layers(newImage, 0)
# scale to output size before cutting black
        pdb.gimp_layer_scale(lay1,outwidth,outheight,False)
        pdb.gimp_layer_set_offsets(lay1,0,0) # after scale, re-position layer.
# cut black 
        pdb.gimp_image_select_color(newImage,0,lay1, black)
        pdb.gimp_edit_cut(lay1)
        pdb.gimp_selection_none(newImage)
# razor cut
        pdb.gimp_image_select_item(newImage,0,lay1)
        pdb.gimp_selection_shrink(newImage,1)
        pdb.gimp_edit_cut(lay1)
# check for all black subject
        non_empty, x1, y1, x2, y2 = pdb.gimp_selection_bounds(newImage)
        pdb.gimp_selection_none(newImage)
        if non_empty:
            None
        else:
            pdb.gimp_message('CAN NOT USE ALL BLACK SUBJECT-\nOr Morph Setting too High on Layer 1')
        pdb.gimp_selection_none(newImage)
# remove highlight from subject  
        pdb.gimp_image_select_item(newImage, 0, oriSUB)
        pdb.gimp_edit_cut(lay1)
        pdb.gimp_selection_none(newImage)
# done - set
        lay1.mode = SCREEN_MODE
        lay1.opacity = 20.0
        pdb.gimp_item_set_visible(lay1,False)
    if stitched != 0: 
        out1a = True
        lay1a = lay1.copy()
        lay1a.mode = NORMAL_MODE
        newImage.add_layer(lay1a, 0)
        #worklay = pdb.gimp_image_get_active_layer(newImage)
        #Solidify_A_Layer(2,worklay)  #was using #2 with good result - should make var
        Solidify_A_Layer(2,lay1a)
        if stitched == 1:
            lay1a.name = "Stitched,full"
        if stitched == 2:
            pdb.plug_in_gmic(newImage, lay1a, 1,      # 1=input only active layer
            "-v - -gimp_boost_fade 9.6,0") 
            lay1a.name = "Stitched,full,boosted"
        if stitched == 3:
            pdb.plug_in_colortoalpha(newImage, lay1a, white) 
            lay1a.name = "Stitched,full,no white"
        if stitched == 4:
            pdb.plug_in_gmic(newImage, lay1a, 1,      # 1=input only active layer
            "-v - -gimp_boost_fade 9.6,0") 
            pdb.plug_in_colortoalpha(newImage, lay1a, white) 
            lay1a.name = "Stitched,full,boosted, no white"
        if stitched == 5:
            pdb.gimp_image_select_item(newImage, 0, oriSUB)
            pdb.gimp_edit_cut(lay1a)
            pdb.gimp_selection_none(newImage)
            lay1a.name = "Stitched,cut"
        if stitched == 6:
            pdb.gimp_image_select_item(newImage, 0, oriSUB)
            pdb.gimp_edit_cut(lay1a)
            pdb.gimp_selection_none(newImage)
            pdb.plug_in_gmic(newImage, lay1a, 1,      # 1=input only active layer
            "-v - -gimp_boost_fade 9.6,0")
            lay1a.name = "Stitched,cut,boosted"
        if stitched == 7:
            pdb.gimp_image_select_item(newImage, 0, oriSUB)
            pdb.gimp_edit_cut(lay1a)
            pdb.gimp_selection_none(newImage)
            pdb.plug_in_colortoalpha(newImage, lay1a, white)
            lay1a.name = "Stitched,cut,no white"
        if stitched == 8:
            pdb.gimp_image_select_item(newImage, 0, oriSUB)
            pdb.gimp_edit_cut(lay1a)
            pdb.gimp_selection_none(newImage)
            pdb.plug_in_gmic(newImage, lay1a, 1,      # 1=input only active layer
            "-v - -gimp_boost_fade 9.6,0")
            pdb.plug_in_colortoalpha(newImage, lay1a, white)
            lay1a.name = "Stitched,cut,boosted,no white"
# done - set stitched
        lay1a.mode = SCREEN_MODE
        lay1a.opacity = 35.0
        pdb.gimp_item_set_visible(lay1a, False)
        pdb.gimp_image_raise_item(newImage, lay1)

####### Layer 2 - Highlight Layer with low intensity   ##################################################
    LayerString = '2x'+lay2str
    Decode_A_Layer(LayerString)
    if makeLayer:
        out2 = True
        lay2 = oriSUB.copy()
        lay2.mode = NORMAL_MODE
        lay2.name = "HL Low"
        lay2.opacity = 100.0 
        newImage.add_layer(lay2, 0)
        pdb.gimp_item_set_visible(lay2, False)
# scale with layerScale from LayerString
        if layerScale != workScale:
            pdb.gimp_layer_scale_full(lay2,(workwidth*layerScale),(workheight*layerScale), True,3) # True = local origin, 3=Lanczos
            pdb.gimp_layer_set_offsets(lay2,0,0) # after scale, re-position layer.
# solidify
        if not solidifyOption == 0:
            Solidify_A_Layer(solidifyOption,lay2)
        if deSat:
            pdb.gimp_desaturate_full(lay2,2)
# squiggle highlight at low scale
        pdb.plug_in_gmic(newImage, lay2, 1,    # 1=input only active layer 
        "-v - -gimp_Squiggly "+str(squiggleNoise)+","+str(segmentLength)+",3.5,0,.6,0,0,0,3,0.2,0.4,12,1,1,0")
        #tested lay2.name after squiggle still has same layername
# scale to worksize (worksize) if layerScale != workScale
        if layerScale != workScale:
            pdb.gimp_layer_scale(lay2,workwidth,workheight,False)
            pdb.gimp_layer_set_offsets(lay2,0,0) # after scale, re-position layer.
# morph
        pdb.plug_in_gmic(newImage, lay2, 1,    # 1=input only active layer 
        '-v - -gimp_morpho_v2 0,0,'+str(morphAmount)+','+'"1,0,1; 0,1,0; 1,0,1",0,0,0,1,0')
# intensify custom
        pdb.gimp_item_set_visible(lay2,True)
        Intensify_Custom(lay2,5)
        lay2 = pdb.gimp_image_merge_visible_layers(newImage,0)
# scale to output size before cutting black
        pdb.gimp_layer_scale(lay2,outwidth,outheight,False)
        pdb.gimp_layer_set_offsets(lay2,0,0) # after scale, re-position layer.
# cut black        
        pdb.gimp_image_select_color(newImage,0,lay2,black)
        pdb.gimp_edit_cut(lay2)
        pdb.gimp_selection_none(newImage)
# remove highlight from subject  
        pdb.gimp_image_select_item(newImage, 0, oriSUB)
        pdb.gimp_edit_cut(lay2)
        pdb.gimp_selection_none(newImage)
# done - set 
        lay2.mode = SCREEN_MODE
        lay2.opacity = 20.0
        pdb.gimp_item_set_visible(lay2,False)
# add weird layers if chosen
        if weird:
            out2a = True
            lay2a = lay2.copy()
            newImage.add_layer(lay2a, 0)
            pdb.gimp_image_select_item(newImage,0,lay2a)
            pdb.gimp_selection_shrink(newImage,1)
            pdb.gimp_edit_cut(lay2a)
            pdb.gimp_selection_none(newImage)
# remove highlight from subject  
            pdb.gimp_image_select_item(newImage, 0, oriSUB)
            pdb.gimp_edit_cut(lay2a)
            pdb.gimp_selection_none(newImage)
# done - set weird
            lay2a.mode = SCREEN_MODE
            lay2a.opacity = 35.0
            lay2a.name = 'HL Low Weird'
            pdb.gimp_item_set_visible(lay2a, False)

####### Layer 3 Highlight Layer with Intensity ####################################################
    LayerString = '3x'+lay3str
    Decode_A_Layer(LayerString)
    if makeLayer:
        out3 = True
        lay3 = oriSUB.copy()
        lay3.mode = NORMAL_MODE
        lay3.name = "HL Intense"
        lay3.opacity = 100.0 
        newImage.add_layer(lay3, 0)
        pdb.gimp_item_set_visible(lay3, False)
# scale with layerScale from LayerString
        if layerScale != workScale:
            pdb.gimp_layer_scale_full(lay3,(workwidth*layerScale),(workheight*layerScale), True,3) # True = local origin, 3=Lanczos
            pdb.gimp_layer_set_offsets(lay3,0,0) # after scale, re-position layer.
# solidify
        if not solidifyOption == 0:
            Solidify_A_Layer(solidifyOption,lay3)
        if deSat:
            pdb.gimp_desaturate_full(lay3,2)
# squiggle
        pdb.plug_in_gmic(newImage, lay3, 1,    # 1=input only active layer 
        "-v - -gimp_Squiggly "+str(squiggleNoise)+","+str(segmentLength)+",3.27,0,.5,0,0,0,3,0.2,0.4,12,1,1,0")
       # pdb.plug_in_gmic(newImage, lay3, 1,    # 1=input only active layer -- custom set for highlight 3
# scale to work size if layerScale != workScale
        if layerScale != workScale:
            pdb.gimp_layer_scale(lay3,workwidth,workheight,False)
            pdb.gimp_layer_set_offsets(lay3,0,0) # after scale, re-position layer.
# morph
        pdb.plug_in_gmic(newImage, lay3, 1,    # 1=input only active layer 
        '-v - -gimp_morpho_v2 0,0,'+str(morphAmount)+','+'"1,0,1; 0,1,0; 1,0,1",0,0,0,1,0')
        #pdb.plug_in_gmic(newImage, lay3, 1,    # 1=input only active layer
        #'-v - -gimp_morpho_v2 0,0,'+str(morphAmount)+','+'"1,0,1; 0,1,0; 1,0,1",0,0,0,1,0')
# intensify inTens
        pdb.gimp_item_set_visible(lay3,True)
        Intensify_inTens(lay3,inTens)
        lay3 = pdb.gimp_image_merge_visible_layers(newImage,0)
# scale to output size before cutting black
        pdb.gimp_layer_scale(lay3,outwidth,outheight,False)
        pdb.gimp_layer_set_offsets(lay3,0,0) # after scale, re-position layer.
# cut black
        pdb.gimp_image_select_color(newImage,0,lay3,black)
        pdb.gimp_edit_cut(lay3)
        pdb.gimp_selection_none(newImage)
# remove highlight from subject  
        pdb.gimp_image_select_item(newImage, 0, oriSUB)
        pdb.gimp_edit_cut(lay3)
        pdb.gimp_selection_none(newImage)
# done - set
        lay3.mode = SCREEN_MODE
        lay3.opacity = 35.0
        pdb.gimp_item_set_visible(lay3,False)

####### Layer 4  HL Layer Thick  ##############################################################
    LayerString = '4x'+lay4str
    Decode_A_Layer(LayerString)
    if makeLayer:
        out4 = True
        lay4 = oriSUB.copy()
        lay4.mode = NORMAL_MODE
        lay4.name = "HL Wide Intense"
        lay4.opacity = 100.0 
        newImage.add_layer(lay4, 0)
        pdb.gimp_item_set_visible(lay4, False)
# scale with layerScale from LayerString
        if layerScale != workScale:
            pdb.gimp_layer_scale_full(lay4,(workwidth*layerScale),(workheight*layerScale), True,3) # True = local origin, 3=Lanczos
            pdb.gimp_layer_set_offsets(lay4,0,0) # after scale, re-position layer.
# solidify
        if not solidifyOption == 0:
            Solidify_A_Layer(solidifyOption,lay4)
        if deSat:
            pdb.gimp_desaturate_full(lay4,2)
# squiggle
        pdb.gimp_image_set_active_layer(newImage,lay4) # verify active for gmic call
        pdb.plug_in_gmic(newImage, lay4, 1,    # 1=input only active layer 
        "-v - -gimp_Squiggly "+str(squiggleNoise)+","+str(segmentLength)+",3.27,0,.5,0,0,0,3,0.2,0.4,12,1,1,0")
# scale to work size  if layerScale != workScale
        if layerScale != workScale:
            pdb.gimp_layer_scale(lay4,workwidth,workheight,False)
            pdb.gimp_layer_set_offsets(lay4,0,0) # after scale, re-position layer.
# morph
        pdb.plug_in_gmic(newImage, lay4, 1,    # 1=input only active layer 
        '-v - -gimp_morpho_v2 0,0,'+str(morphAmount)+','+'"1,0,1; 0,1,0; 1,0,1",0,0,0,1,0')
# intensify inTens
        pdb.gimp_item_set_visible(lay4,True)
        Intensify_inTens(lay4,inTens)
        lay4 = pdb.gimp_image_merge_visible_layers(newImage,0)
# scale to output size before cutting black
        pdb.gimp_layer_scale(lay4,outwidth,outheight,False)
        pdb.gimp_layer_set_offsets(lay4,0,0) # after scale, re-position layer.
# cut black
        pdb.gimp_image_select_color(newImage,0,lay4,black)
        pdb.gimp_edit_cut(lay4)
        pdb.gimp_selection_none(newImage)
# remove highlight from subject  
        pdb.gimp_image_select_item(newImage, 0, oriSUB)
        pdb.gimp_edit_cut(lay4)
        pdb.gimp_selection_none(newImage)
# done - set
        lay4.mode = SCREEN_MODE
        lay4.opacity = 20.0
        pdb.gimp_item_set_visible(lay4,False)
# add weird layers if chosen
        if weird:
            out4a = True
            lay4a = lay4.copy()
            newImage.add_layer(lay4a, 0)
            pdb.gimp_image_select_item(newImage,0,lay4a)
            pdb.gimp_selection_shrink(newImage,1)
            pdb.gimp_edit_cut(lay4a)
            pdb.gimp_selection_none(newImage)
# remove highlight from subject  
            pdb.gimp_image_select_item(newImage, 0, oriSUB)
            pdb.gimp_edit_cut(lay4a)
            pdb.gimp_selection_none(newImage)
# done - set weird
            lay4a.mode = SCREEN_MODE  
            lay4a.opacity = 35.0
            lay4a.name = 'HL Wide Weird'
            pdb.gimp_item_set_visible(lay4a, False)

####### Layer 5 - Kirlian A ###################################################################
    LayerString = '5x'+lay5str   ### tested using  lay6str : same as with lay5str
    Decode_A_Layer(LayerString)
    if makeLayer:
        out5 = True
        lay5 = oriSUB.copy()
        lay5.mode = NORMAL_MODE
        lay5.name = "Kirlian A" 
        lay5.opacity = 100.0 
        newImage.add_layer (lay5, 0)
        pdb.gimp_item_set_visible(lay5, False)
# special hardcoded scale down for better solidify workwidth*.2
        if not solidifyOption == 0:
            pdb.gimp_layer_scale_full(lay5,int(workwidth*.2),int(workheight*.2), True,3) # True = local origin, 3=Lanczos
            pdb.gimp_layer_set_offsets(lay5,0,0) # after scale, re-position layer.
# solidify
        if not solidifyOption == 0:
            Solidify_A_Layer(solidifyOption,lay5)
        if deSat:
            pdb.gimp_desaturate_full(lay5,2)
# special hardcoded scale for squiggle in layer 5
        pdb.gimp_layer_scale_full(lay5,int(workwidth*.8),int(workheight*.8), True,3) # True = local origin, 3=Lanczos
        pdb.gimp_layer_set_offsets(lay5,0,0) # after scale, re-position layer.
# squiggle at 80% worksize
        pdb.plug_in_gmic(newImage, lay5, 1,    # 1=input active layer   ",1.80,0,0.5,0,0,0,2.5,0.2,0.4,12,1,1,0")
        "-v - -gimp_Squiggly "+str(squiggleNoise)+","+str(segmentLength)+",1.8,0,0.5,0,0,0,3,0.2,0.4,12,1,1,0")
# scale to worksize for morph == different by design.
        pdb.gimp_layer_scale_full(lay5,int(workwidth),int(workheight), True,3) # True = local origin, 3=Lanczos
        pdb.gimp_layer_set_offsets(lay5,0,0) # after scale, re-position layer.
# morph at worksize
        pdb.plug_in_gmic(newImage, lay5, 1,    # 1=input only active layer, 0=output in place
        '-v - -gimp_morpho_v2 0,0,'+str(morphAmount)+','+'"1,0,1; 0,1,0; 1,0,1",0,0,0,1,0')
# intensify inTens
        pdb.gimp_item_set_visible(lay5,True)
        Intensify_inTens(lay5,inTens)
        lay5 = pdb.gimp_image_merge_visible_layers(newImage,0)
# scale to output size before cutting black
        pdb.gimp_layer_scale(lay5,outwidth,outheight,False)
        pdb.gimp_layer_set_offsets(lay5,0,0) # after scale, re-position layer.
# cut black
        pdb.gimp_image_select_color(newImage,0,lay5,black)
        pdb.gimp_edit_cut(lay5)
        pdb.gimp_selection_none(newImage)
####### split into object and background layers  
        lay5bg = lay5.copy()
        lay5bg.mode = NORMAL_MODE
        lay5bg.name = "Kirlian A BG" 
        lay5bg.opacity = 100.0 
        newImage.add_layer (lay5bg, 0)
        pdb.gimp_item_set_visible(lay5bg, False)            
        pdb.gimp_image_select_item(newImage, 0, oriSUB)
        pdb.gimp_edit_cut(lay5bg)
        pdb.gimp_selection_invert(newImage)
        pdb.gimp_edit_cut(lay5)
        lay5.name = "Kirlian A Subject"
        pdb.gimp_selection_none(newImage)        
# done - set
        lay5.mode = SCREEN_MODE
        lay5.opacity = 35.0
        pdb.gimp_item_set_visible(lay5,False)
        lay5bg.mode = SCREEN_MODE
        lay5bg.opacity = 35.0
        pdb.gimp_item_set_visible(lay5bg,False)

####### Layer 6  Kirlian B with Intensity  ###########################################################
    LayerString = '6x'+lay6str
    Decode_A_Layer(LayerString)
    if makeLayer:
        out6 = True
        lay6 = oriSUB.copy()
        lay6.mode = NORMAL_MODE
        lay6.name = "Kirlian B" 
        lay6.opacity = 100.0 
        newImage.add_layer (lay6, 0)
        pdb.gimp_item_set_visible(lay6, False)
# special hardcoded scale down for better solidify workwidth*.2
        if not solidifyOption == 0:
            pdb.gimp_layer_scale_full(lay6,int(workwidth*.2),int(workheight*.2), True,3) # True = local origin, 3=Lanczos
            pdb.gimp_layer_set_offsets(lay6,0,0) # after scale, re-position layer.
# solidify
            Solidify_A_Layer(solidifyOption,lay6)
        if deSat:
            pdb.gimp_desaturate_full(lay6,2)
# special hardcoded scale for squiggle in layer 6
        pdb.gimp_layer_scale_full(lay6,int(workwidth*.8),int(workheight*.8), True,3) # True = local origin, 3=Lanczos
        pdb.gimp_layer_set_offsets(lay6,0,0) # after scale, re-position layer.
# squiggle at 80% worksize
        pdb.plug_in_gmic(newImage, lay6, 1,    # 1=input only active layer 
        "-v - -gimp_Squiggly "+str(squiggleNoise)+","+str(segmentLength)+",1.8,0,0.5,0,0,0,3,0.2,0.4,12,1,1,0")
# scale to worksize for morph == different by design.
        pdb.gimp_layer_scale_full(lay6,int(workwidth),int(workheight), True,3) # True = local origin, 3=Lanczos
        pdb.gimp_layer_set_offsets(lay6,0,0) # after scale, re-position layer.
# morph at worksize
        pdb.plug_in_gmic(newImage, lay6, 1,    # 1=input only active layer 
        '-v - -gimp_morpho_v2 0,0,'+str(morphAmount)+','+'"1,0,1; 0,1,0; 1,0,1",0,0,0,1,0')
# intensify inTens
        pdb.gimp_item_set_visible(lay6,True)
        Intensify_inTens(lay6,inTens)
        lay6 = pdb.gimp_image_merge_visible_layers(newImage,0)
# scale to output size before cutting black
        pdb.gimp_layer_scale(lay6,outwidth,outheight,False)
        pdb.gimp_layer_set_offsets(lay6,0,0) # after scale, re-position layer.
# cut black
        pdb.gimp_image_select_color(newImage,0,lay6,black)
        pdb.gimp_edit_cut(lay6)
        pdb.gimp_selection_none(newImage)
####### split into object and background layers  
        lay6bg = lay6.copy()
        lay6bg.mode = NORMAL_MODE
        lay6bg.name = "Kirlian B BG" 
        lay6bg.opacity = 100.0 
        newImage.add_layer (lay6bg, 0)
        pdb.gimp_item_set_visible(lay6bg, False)            
        pdb.gimp_image_select_item(newImage, 0, oriSUB)
        pdb.gimp_edit_cut(lay6bg)
        pdb.gimp_selection_invert(newImage)
        pdb.gimp_edit_cut(lay6)
        lay6.name = "Kirlian B Subject"
        pdb.gimp_selection_none(newImage)        
# done - set
        lay6.mode = SCREEN_MODE
        lay6.opacity = 35.0
        pdb.gimp_item_set_visible(lay6,False)
        lay6bg.mode = SCREEN_MODE
        lay6bg.opacity = 35.0
        pdb.gimp_item_set_visible(lay6bg,False)

####### Layer 7 Kirlian C with Intensity #################################################
    LayerString = '7x'+lay7str
    Decode_A_Layer(LayerString)
    if makeLayer:
        out7 = True
        lay7 = oriSUB.copy()
        lay7.mode = NORMAL_MODE
        lay7.name = "Kirlian C" 
        lay7.opacity = 100.0 
        newImage.add_layer (lay7, 0)
        pdb.gimp_item_set_visible(lay7, False)
# special hardcoded scale down for better solidify workwidth*.2
        if not solidifyOption == 0:
            pdb.gimp_layer_scale_full(lay7,int(workwidth*.2),int(workheight*.2), True,3) # True = local origin, 3=Lanczos
            pdb.gimp_layer_set_offsets(lay7,0,0) # after scale, re-position layer.
# solidify
            Solidify_A_Layer(solidifyOption,lay7)
        if deSat:
            pdb.gimp_desaturate_full(lay7,2)
# special hardcoded scale for squiggle in layer 7
        if layerScale != workScale:
            pdb.gimp_layer_scale_full(lay7,int(workwidth*.8),int(workheight*.8), True,3) # True = local origin, 3=Lanczos
            pdb.gimp_layer_set_offsets(lay7,0,0) # after scale, re-position layer.
# squiggle at 80% worksize
        pdb.plug_in_gmic(newImage, lay7, 1,    # 1=input only active layer 
        "-v - -gimp_Squiggly "+str(squiggleNoise)+","+str(segmentLength)+",1.8,0,0.5,0,0,0,3,0.2,0.4,12,1,1,0")
# scale to worksize for morph == different by design.
        pdb.gimp_layer_scale_full(lay7,int(workwidth),int(workheight), True,3) # True = local origin, 3=Lanczos
        pdb.gimp_layer_set_offsets(lay7,0,0) # after scale, re-position layer.
# morph at worksize
        pdb.plug_in_gmic(newImage, lay7, 1,    # 1=input only active layer 
        '-v - -gimp_morpho_v2 0,0,'+str(morphAmount)+','+'"1,0,1; 0,1,0; 1,0,1",0,0,0,1,0')
# intensify inTens
        pdb.gimp_item_set_visible(lay7,True)
        Intensify_inTens(lay7,inTens)
        lay7 = pdb.gimp_image_merge_visible_layers(newImage,0)
# scale to output size before cutting black
        pdb.gimp_layer_scale(lay7,outwidth,outheight,False)
        pdb.gimp_layer_set_offsets(lay7,0,0) # after scale, re-position layer.
# cut black
        pdb.gimp_image_select_color(newImage,0,lay7,black)
        pdb.gimp_edit_cut(lay7)
        pdb.gimp_selection_none(newImage)
####### split into object and background layers  
        lay7bg = lay7.copy()
        lay7bg.mode = NORMAL_MODE
        lay7bg.name = "Kirlian C BG" 
        lay7bg.opacity = 100.0 
        newImage.add_layer (lay7bg, 0)
        pdb.gimp_item_set_visible(lay7bg, False)            
        pdb.gimp_image_select_item(newImage, 0, oriSUB)
        pdb.gimp_edit_cut(lay7bg)
        pdb.gimp_selection_invert(newImage)
        pdb.gimp_edit_cut(lay7)
        lay7.name = "Kirlian C Subject"
        pdb.gimp_selection_none(newImage)        
# done - set
        lay7.mode = SCREEN_MODE
        lay7.opacity = 35.0
        pdb.gimp_item_set_visible(lay7,False)
        lay7bg.mode = SCREEN_MODE
        lay7bg.opacity = 35.0
        pdb.gimp_item_set_visible(lay7bg,False)
####### raise BG layers to organize the layer stack
    if out5:
        pdb.gimp_image_raise_item_to_top(newImage,lay5bg)
    if out6:
        pdb.gimp_image_raise_item_to_top(newImage,lay6bg)
    if out7:
        pdb.gimp_image_raise_item_to_top(newImage,lay7bg)

####### Layer 8  Smokey Subject Border  ###########################################################
    if smokey !=0:
        out8 = True
        lay8 = oriSUB.copy()
        lay8.mode = NORMAL_MODE
        lay8.name = "Light Smokey Border" 
        lay8.opacity = 100.0 
        newImage.add_layer(lay8, 0)
        pdb.gimp_item_set_visible(lay8,False)
# do whirl on subject copy 
        pdb.gimp_image_select_item(newImage, 0, lay8)
        pdb.plug_in_gmic(newImage, lay8, 1,      # 1=input only active layer
        '-v - -draw_whirl 20,0')
# do shrink feather and solidify x 2       
        pdb.gimp_selection_shrink(newImage, 10)#<-- testing
        pdb.gimp_selection_feather(newImage, 45)
        pdb.plug_in_gmic(newImage, lay8, 1,      # 1=input only active layer
        '-v - -gimp_solidify_td 75,1,20,4,0')
        pdb.gimp_selection_none(newImage)
        pdb.gimp_image_select_item(newImage, 0, lay8)
        pdb.gimp_selection_feather(newImage, 40)
        pdb.plug_in_gmic(newImage, lay8, 1,      # 1=input only active layer
        '-v - -gimp_solidify_td 75,1,20,4,0')
# set 8a (after make) Light Smokey Subject
        pdb.gimp_curves_spline(lay8, 0, 4,(0,255,255,255))
        lay8a = lay8.copy()
        lay8a.mode = SCREEN_MODE
        lay8a.name = "Light Smokey Subject" 
        lay8a.opacity = 35.0
        newImage.add_layer(lay8a, 0)
# leave subject selection active and squiggle
        pdb.plug_in_gmic(newImage, lay8, 1,    # 1=input only active layer 
        '-v - -gimp_Squiggly 20,15,1.41,0,0.37,0,0,0,4,1,0.68,12,1,1,0')
        pdb.gimp_image_set_active_layer(newImage, lay8)
# no morph on the smokey layers
# intensify inTens
        pdb.gimp_item_set_visible(lay8,True)
        Intensify_inTens(lay8,inTens)
        lay8 = pdb.gimp_image_merge_visible_layers(newImage,0)
# scale to output size before cutting black
        pdb.gimp_layer_scale(lay8,outwidth,outheight,False)
        pdb.gimp_layer_set_offsets(lay8,0,0) # after scale, re-position layer.
# cut black
        pdb.gimp_image_select_color(newImage,0,lay8,black)
        pdb.gimp_edit_cut(lay8)
        pdb.gimp_selection_none(newImage)
# done 8 - set light smokey Light Smokey Border
        pdb.gimp_invert(lay8)
        pdb.gimp_curves_spline(lay8, 0, 4,(0,255,255,255))
        lay8.mode = SCREEN_MODE
        lay8.opacity = 20.0
        pdb.gimp_item_set_visible(lay8,False)
# add dark smokey border 
        lay8b = lay8.copy()
        lay8b.mode = NORMAL_MODE
        lay8b.name = "Dark Smokey Border"
        lay8b.opacity = 100.0
        newImage.add_layer(lay8b,0)
# invert colors for dark smokey border
        pdb.gimp_invert(lay8b)
        pdb.gimp_curves_spline(lay8b, 0, 4,(0,0,255,0))
# intensify  
#        lay8b.mode = NORMAL_MODE
#        pdb.gimp_item_set_visible(lay8b,True)
#        Intensify_inTens(lay8b,inTens)
# done 8b - set dark smokey border
        lay8b.mode = GRAIN_MERGE_MODE
        lay8b.opacity = 15.0
        pdb.gimp_item_set_visible(lay8b,False)
# add dark smokey subject 
        lay8c = lay8a.copy()
# done 8c - set dark smokey subject
        lay8c.mode = GRAIN_MERGE_MODE
        lay8c.name = "Dark Smokey Subject"
        lay8c.opacity = 20.0
        newImage.add_layer(lay8c,0)
        pdb.gimp_curves_spline(lay8c, 0, 4,(0,0,255,0))

####### Layer 9 --- Sparkle Layer < intermittant relative to size, subject etc.. #######################
    LayerString = '9x'+lay9str
    Decode_A_Layer(LayerString)
    if makeLayer:
        out9 = True
        lay9 = oriSUB.copy()
        lay9.mode = NORMAL_MODE
        lay9.name = "Sparkle" 
        lay9.opacity = 100.0 
        newImage.add_layer (lay9, 0)
        pdb.gimp_item_set_visible(lay9, False)
# special scale down for better solidify @ 20% 
        if not solidifyOption == 0:
            pdb.gimp_layer_scale_full (lay9,int(workwidth*.2),int(workheight*.2), True,3) # True = local origin, 3=Lanczos
            pdb.gimp_layer_set_offsets(lay9,0,0) # after scale, re-position layer.
# solidify
            Solidify_A_Layer(solidifyOption,lay9)
        if deSat:
            pdb.gimp_desaturate_full(lay9,2)
# scale for squiggle & morph
        pdb.gimp_layer_scale(lay9,(workwidth*layerScale),(workheight*layerScale),False)
        pdb.gimp_layer_set_offsets(lay9,0,0) # after scale, re-position layer.
# squiggle
        pdb.plug_in_gmic(newImage, lay9, 1,    # 1=input only active layer 
        "-v - -gimp_Squiggly "+str(squiggleNoise)+","+str(segmentLength)+",1.8,0,0.5,0,0,0,3,0.2,0.4,12,1,1,0")
# morph
        pdb.plug_in_gmic(newImage, lay9, 1,    # 1=input only active layer 
        '-v - -gimp_morpho_v2 0,0,'+str(morphAmount)+','+'"1,0,1; 0,1,0; 1,0,1",0,0,0,1,0')
# scale back to worksize
        pdb.gimp_layer_scale(lay9,workwidth,workheight,False)
        pdb.gimp_image_resize_to_layers(newImage)
        pdb.gimp_layer_set_offsets(lay9,0,0) # after scale, re-position layer.
# intensify inTens
        pdb.gimp_item_set_visible(lay9,True)
        Intensify_inTens(lay9,inTens)
        lay9 = pdb.gimp_image_merge_visible_layers(newImage,0)
# scale to output size before cutting black
        pdb.gimp_layer_scale(lay9,outwidth,outheight,False)
        pdb.gimp_layer_set_offsets(lay9,0,0) # after scale, re-position layer.
# cut black
        pdb.gimp_image_select_color(newImage,0,lay9,black)
        pdb.gimp_edit_cut(lay9)
        pdb.gimp_selection_none(newImage)
# done - set 
        lay9.mode = SCREEN_MODE
        lay9.opacity = 35.0
        pdb.gimp_item_set_visible(lay9,False)

####### Layer 10  My Layer  #########################################################
    LayerString = '10x'+lay10str
    Decode_A_Layer(LayerString)
    if makeLayer:
        out10 = True
        lay10 = oriSUB.copy()
        lay10.mode = NORMAL_MODE
        lay10.name = "My Layer" 
        lay10.opacity = 100.0 
        newImage.add_layer (lay10, 0)
        pdb.gimp_item_set_visible(lay10, False)
# special hardcoded scale down for better solidify workwidth*.2
        if not solidifyOption == 0:
            pdb.gimp_layer_scale_full(lay10,int(workwidth*.2),int(workheight*.2), True,3) # True = local origin, 3=Lanczos
            pdb.gimp_layer_set_offsets(lay10,0,0) # after scale, re-position layer.
# solidify
            Solidify_A_Layer(solidifyOption,lay10)
        if deSat:
            pdb.gimp_desaturate_full(lay10,2)
# special hardcoded scale for squiggle in layer 10
        pdb.gimp_layer_scale_full(lay10,int(workwidth*(layerScale)),int(workheight*(layerScale)), True,3) # True = local origin, 3=Lanczos
        pdb.gimp_layer_set_offsets(lay10,0,0) # after scale, re-position layer.
# squiggle at 80% worksize
        pdb.plug_in_gmic(newImage, lay10, 1,    # 1=input only active layer 
        "-v - -gimp_Squiggly "+str(squiggleNoise)+","+str(segmentLength)+",1.8,0,0.5,0,0,0,3,0.2,0.4,12,1,1,0")
# scale to worksize for morph == different by design.
        pdb.gimp_layer_scale_full(lay10,int(workwidth*(layerScale)),int(workheight*(layerScale)), True,3) # True = local origin, 3=Lanczos
        pdb.gimp_layer_set_offsets(lay10,0,0) # after scale, re-position layer.
# morph at worksize
        pdb.plug_in_gmic(newImage, lay10, 1,    # 1=input only active layer 
        '-v - -gimp_morpho_v2 0,0,'+str(morphAmount)+','+'"1,0,1; 0,1,0; 1,0,1",0,0,0,1,0')
# intensify inTens
        pdb.gimp_item_set_visible(lay10,True)
        Intensify_inTens(lay10,inTens)
        lay10 = pdb.gimp_image_merge_visible_layers(newImage,0)
# scale to output size before cutting black
        pdb.gimp_layer_scale(lay10,outwidth,outheight,False)
        pdb.gimp_layer_set_offsets(lay10,0,0) # after scale, re-position layer.
# cut black
        pdb.gimp_image_select_color(newImage,0,lay10,black)
        pdb.gimp_edit_cut(lay10)
        pdb.gimp_selection_none(newImage)
####### split into object and background layers  
        lay10bg = lay10.copy()
        lay10bg.mode = NORMAL_MODE
        lay10bg.name = "My Laner BG" 
        lay10bg.opacity = 100.0 
        newImage.add_layer (lay10bg, 0)
        pdb.gimp_item_set_visible(lay10bg, False)            
        pdb.gimp_image_select_item(newImage, 0, oriSUB)
        pdb.gimp_edit_cut(lay10bg)
        pdb.gimp_selection_invert(newImage)
        pdb.gimp_edit_cut(lay10)
        lay10.name = "My Layer"
        pdb.gimp_selection_none(newImage)        
# done - set
        lay10.mode = SCREEN_MODE
        lay10.opacity = 35.0
        pdb.gimp_item_set_visible(lay10,False)
        lay10bg.mode = SCREEN_MODE
        lay10bg.opacity = 35.0
        pdb.gimp_item_set_visible(lay10bg,False)


####### Add Bubbles Background (button choice) ################################################
    if addbub !=0:
        out20 = True
        lay20= oriSUB.copy()
        lay20.mode = NORMAL_MODE
        lay20.name = "Bubbles" 
        lay20.opacity = 100.0 
        newImage.add_layer (lay20, 0)
        pdb.gimp_item_set_visible(lay20, False)
# special scale down for better solidify @ 20% 
        pdb.gimp_layer_scale_full (lay20,int(workwidth*.2),int(workheight*.2), True,3) # True = local origin, 3=Lanczos
        pdb.gimp_layer_set_offsets(lay20,0,0) # after scale, re-position layer.
# solidify
        pdb.plug_in_gmic(newImage, lay20, 1,   "-v - -gimp_solidify_td 50,1,0,0,0")
# squiggle while small
        pdb.plug_in_gmic(newImage, lay20, 1,    # 1=input only active layer 
        "-v - -gimp_Squiggly 2,4,1.8,0,0.5,0,0,0,3,0.2,0.4,12,1,1,0")
# scale for morph
        ## issue with layerScale = 2 since no layerstring here... also hard coded next scale....
        pdb.gimp_layer_scale(lay20,(workwidth*2),(workheight*2),False)
        pdb.gimp_layer_set_offsets(lay20,0,0) # after scale, re-position layer.
# morph
        pdb.plug_in_gmic(newImage, lay20, 1,    # 1=input only active layer --
        '-v - -gimp_morpho_v2 0,0,2,"1,0,1; 0,1,0; 1,0,1",0,0,0,1,0')
# scale back to work size if layerScale != workScale
        if 2 != workScale:
            pdb.gimp_layer_scale(lay20,workwidth,workheight,False)
            pdb.gimp_image_resize_to_layers(newImage)
            pdb.gimp_layer_set_offsets(lay20,0,0) # after scale, re-position layer.
# intensify custom
        pdb.gimp_item_set_visible(lay20,True)
        Intensify_Custom(lay20,40)
        lay20 = pdb.gimp_image_merge_visible_layers(newImage,0)
# multiply intensify custom
        pdb.gimp_item_set_visible(lay20,True)
        Intensify_Custom(lay20,5)
        lay20 = pdb.gimp_image_merge_visible_layers(newImage,0)
# scale to output size before cutting black
        pdb.gimp_layer_scale(lay20,outwidth,outheight,False)
        pdb.gimp_layer_set_offsets(lay20,0,0) # after scale, re-position layer.
# cut black
        pdb.gimp_image_select_color(newImage,0,lay20,black)
        pdb.gimp_edit_cut(lay20)
        pdb.gimp_selection_none(newImage)
# shrink cut for 'bubbles'  
        pdb.gimp_image_select_item(newImage,0,lay20)        
        pdb.gimp_selection_shrink(newImage,1)
        pdb.gimp_edit_cut(lay20)
        pdb.gimp_image_select_item(newImage,0,oriSUB)
        pdb.gimp_edit_cut(lay20)
        pdb.gimp_selection_none(newImage)
# add color to bubbles layer 
        if addbub == 2 or addbub == 3:
            pdb.gimp_item_set_visible(padLayer,True)
            pdb.plug_in_gmic(newImage, lay20, 1,    # 1=input only active layer --
            '-v - -gimp_boost_fade 9.6,0') 
            lay20.name = "Bubbles Boosted"
        if addbub == 3:
            lay20x= lay20.copy()
            newImage.add_layer (lay20x, 0)
            pdb.gimp_item_set_visible(lay20x, True)
            lay20.mode  = SCREEN_MODE
            lay20x.mode = SCREEN_MODE
            lay20x.opacity = 100.0 
            lay20y= lay20x.copy()
            newImage.add_layer (lay20y, 0)
            lay20y.mode = SCREEN_MODE
            lay20y.opacity = 100.0
            pdb.gimp_item_set_visible(lay20y, True)
            pdb.gimp_item_set_visible(lay20, True)
            pdb.gimp_item_set_visible(lay20x,True)
            lay20x = pdb.gimp_image_merge_down(newImage,lay20y,0)
            lay20x.mode = SCREEN_MODE
            lay20 = pdb.gimp_image_merge_down(newImage,lay20x,0) 
            lay20.mode = SCREEN_MODE
            lay20x= lay20.copy()
            newImage.add_layer (lay20x, 0)
            lay20x.mode = SCREEN_MODE
            lay20x.opacity = 100.0
            lay20 = pdb.gimp_image_merge_down(newImage,lay20x,0)
            lay20.name = "Bubbles Max Boost"
        lay20.opacity = 35.0 
        lay20.mode  = SCREEN_MODE 
# cust subject from bubble background  
        pdb.gimp_image_select_item(newImage, 0, oriSUB)
        pdb.gimp_edit_cut(lay20)
        pdb.gimp_selection_none(newImage)
# done - set 
        lay20.mode = SCREEN_MODE
        lay20.opacity = 35.0
        pdb.gimp_item_set_visible(padLayer,False)
        pdb.gimp_item_set_visible(lay20,False)

# create 2 top layers to add color to the subject (from oriSub) #####################
    lay21 = oriSUB.copy()
    lay21.mode = COLOR_MODE
    lay21.name = "Subject Color Layer 1"
    lay21.opacity = 100.0 
    newImage.add_layer(lay21, 0)
    pdb.gimp_item_set_visible(lay21, False)
    
    lay22 = oriSUB.copy()
    lay22.mode = GRAIN_MERGE_MODE
    lay22.name = "GrainMerge/HardLight"
    lay22.opacity = 50.0 
    newImage.add_layer(lay22, 0)
    pdb.gimp_item_set_visible(lay22, False)

################# layer nearly done, make sure all scaled, offset & visibility at wanted
# oriSUB layer has always been at output size -- F.Y.I.
# always produce these layers
    pdb.gimp_layer_scale_full(padLayer,outwidth,outheight, True,3)
    pdb.gimp_layer_set_offsets(padLayer,0,0)
    pdb.gimp_layer_set_visible(padLayer,True)
    pdb.gimp_layer_scale_full(oriSUB,outwidth,outheight, True,3)
    pdb.gimp_layer_set_offsets(oriSUB,0,0)
    pdb.gimp_layer_scale_full(lay21,outwidth,outheight, True,3)
    pdb.gimp_layer_set_offsets(lay21,0,0)
    pdb.gimp_item_set_visible(lay21,True)
    pdb.gimp_layer_scale_full(lay22,outwidth,outheight, True,3)
    pdb.gimp_layer_set_offsets(lay22,0,0)
    pdb.gimp_item_set_visible(lay22,True)
# check for optional layers
    if out1:
        pdb.gimp_layer_scale_full(lay1, outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay1,0,0)
        pdb.gimp_item_set_visible(lay1,True)
    if out2:
        pdb.gimp_layer_scale_full(lay2, outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay2,0,0)
        pdb.gimp_item_set_visible(lay2,True)
    if out3:
        pdb.gimp_layer_scale_full(lay3, outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay3,0,0)
        pdb.gimp_item_set_visible(lay3,True)
    if out4:
        pdb.gimp_layer_scale_full(lay4, outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay4,0,0)
        pdb.gimp_item_set_visible(lay4,True)
    if out5: #kirlian A
        pdb.gimp_layer_scale_full(lay5, outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay5,0,0)
        pdb.gimp_item_set_visible(lay5,True)
        pdb.gimp_layer_scale_full(lay5bg, outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay5bg,0,0)
        pdb.gimp_item_set_visible(lay5bg,True)
    if out6: #kirlian B
        pdb.gimp_layer_scale_full(lay6, outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay6,0,0)
        pdb.gimp_item_set_visible(lay6,True)
        pdb.gimp_layer_scale_full(lay6bg, outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay6bg,0,0)
        pdb.gimp_item_set_visible(lay6bg,True)
    if out7: #kirlian C
        pdb.gimp_layer_scale_full(lay7, outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay7,0,0)
        pdb.gimp_item_set_visible(lay7,True)
        pdb.gimp_layer_scale_full(lay7bg, outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay7bg,0,0)
        pdb.gimp_item_set_visible(lay7bg,True)
    if out8:  #smokey layer
        pdb.gimp_layer_scale_full(lay8, outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay8,0,0)
        pdb.gimp_item_set_visible(lay8,False)
        pdb.gimp_layer_scale_full(lay8a, outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay8a,0,0)
        pdb.gimp_item_set_visible(lay8a,False)
        pdb.gimp_layer_scale_full(lay8b, outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay8b,0,0)
        pdb.gimp_item_set_visible(lay8b,False)
        pdb.gimp_layer_scale_full(lay8c, outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay8c,0,0)
        pdb.gimp_item_set_visible(lay8c,False)
        if smokey != 1 and smokey !=0:
            if smokey == 2:  # delete darks
                pdb.gimp_image_remove_layer(newImage, lay8b)
                pdb.gimp_image_remove_layer(newImage, lay8c)
            if smokey == 3:  # delete lights
                pdb.gimp_image_remove_layer(newImage, lay8)
                pdb.gimp_image_remove_layer(newImage, lay8a)
    if out9: #sparkle layer  
        pdb.gimp_layer_scale_full(lay9,outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay9,0,0)
        pdb.gimp_item_set_visible(lay9,True)
    if out10: # your kirlian layer  
        pdb.gimp_layer_scale_full(lay10,outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay10,0,0)
        pdb.gimp_item_set_visible(lay10,True)
    if out20:
        pdb.gimp_layer_scale_full(lay20,outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay20,0,0)
        pdb.gimp_item_set_visible(lay20,True)
    if out1a: # Stitch
        pdb.gimp_layer_scale_full(lay1a,outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay1a,0,0)
        pdb.gimp_item_set_visible(lay1a,True)
    if out2a:
        pdb.gimp_layer_scale_full(lay2a,outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay2a,0,0)
        pdb.gimp_item_set_visible(lay2a,True)
    if out4a:
        pdb.gimp_layer_scale_full(lay4a,outwidth,outheight, True,3)
        pdb.gimp_layer_set_offsets(lay4a,0,0)
        pdb.gimp_item_set_visible(lay4a,True)
    pdb.gimp_image_resize_to_layers(newImage)        

    # Clean Up
    pdb.gimp_display_new (newImage)
    pdb.gimp_displays_flush
   
    # Close the undo group.
    pdb.gimp_context_pop
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)
    
    return()    


# This is the plugin registration function
register(
    "BA_SpiderPaint_v1_179",
    "BA_SpiderPaint_v1qt Layer Line Commands are delimited with X or X\nNeeds G'Mic 1.7.9  \nOutputs user selected layers image creation. \nCreate Layer Y/N x Desaturate Y/N x Solidify Choice 0-20 x \nScale% 10-300 x Noise .0-20.0 x SegLen .5-15.0 x Morph 0-10   ",
    "Tested and works with G'Mic 1.7.9",
    "animicule@gimpchat.com", 
    "Brian Andersen ", 
    "March 2017",
    "BA SpiderPaint v1 Gmic 1.7.9 ...",
    "RGB*",
    [
      (PF_IMAGE,      "image"      , "Input image                                 ", None),   #Y x N x 4 x50x2x 15 x 2
      (PF_DRAWABLE,   "drawable"   , "Input drawable                              ", None),
      (PF_ADJUSTMENT, "widthheight", "Width+Height of Output Image (0 = same as input)",    0, (   0, 9600, 300)),
      (PF_ADJUSTMENT, "worklen"    , "Process size                                ",  2000,(   0, 9600, 300)),
      (PF_SLIDER,     "inTens"     , "Intensity                                   ",   30, (   4,   40,   1)),
      (PF_STRING,     "lay1str"    , " 1 Highlight Razorsplit with Intensity      ", "Y xNx  1 x50x 7   x 12  x 3"),
      (PF_STRING,     "lay2str"    , " 2 Highlight Layer Low Intensity            ", "y XnX 16 x50x 7   x 12  x 2"),
      (PF_STRING,     "lay3str"    , " 3 Highlight Layer with Intensity           ", "Y xNx  3 x50x 7   x 12  x 3"),
      (PF_STRING,     "lay4str"    , " 4 Highlight Layer Thick w/Intensity        ", "Y xNx  4 x50x 2   x 15  x 2"),
      (PF_STRING,     "lay5str"    , " 5 Kirlian A Layers with Intensity          ", "Y xNx  9 x50x 1.5 x 1.5 x 2"),
      (PF_STRING,     "lay6str"    , " 6 Kirlian B Layers with Intensity          ", "Y xNx  2 x50x  2  x 3   x 2"),
      (PF_STRING,     "lay7str"    , " 7 Kirlian C Layers with Intensity          ", "Y xNx 17 x50x  3  x 5   x 2"),
      (PF_OPTION,     "smokey"     , " 8 Add Smokey Border Layers                 ", 0, ("None", "All", "Light", "Dark")),
      (PF_STRING,     "lay9str"    , " 9 Sparkle Layer (may need morph adjustment)", "Y xNx10x150x3x2x 4"), 
      (PF_STRING,     "lay10str"   , "10 Custom Layer - Your choices              ", "N x  Enter your string here  "),
      (PF_OPTION,     "stitch"     , "Add Stitched Background (select option)     ", 0, ("None", "Stitched Full", "Stitched Full Boosted", "Stitched Full No white", "Stitched Full Boosted No White", "Stitched Cut", "Stitched Cut Boosted", "Stitched Cut No white", "Stitched Cut Boosted No White")),     
      (PF_OPTION,     "addbub"     , "Add Bubble Background                       ", 0, ("None", "Normal", "Boost Color", "Max Boost")),
      (PF_TOGGLE,     "weird"      , "Add Extra Cut Highlights into the stack     ", False), 
 

	  
    ],
    [],
    BA_SpiderPaint_v1_179,
    menu="<Image>/BrianNew/",
    )

main()
